local frame = CreateFrame("Frame"); -- Need a frame to respond to events
EVENTS_TO_REGISTER = {"ADDON_LOADED", "COMBAT_LOG_EVENT_UNFILTERED", "PLAYER_ENTERING_WORLD", "PLAYER_LOGOUT", "CHAT_MSG_ADDON", "ENCOUNTER_START", "ENCOUNTER_END", "PLAYER_REGEN_DISABLED", "PLAYER_REGEN_ENABLED"}
for _, v in pairs(EVENTS_TO_REGISTER) do
	frame:RegisterEvent(v)
end

-- Addon
local hf = CreateFrame("Frame",modName,UIParent);
loaded = false

_G.HardFinger = {}
local HardFinger = _G.HardFinger
local options = HardFinger.options
local playerName = UnitName("player")
HardFinger.HardFinger_Config = HardFinger.HardFinger_Config or {}
local P = HardFinger.HardFinger_Config

COMBATLOG_OBJECT_TYPE_PET		= COMBATLOG_OBJECT_TYPE_PET or 0x00001000
COMBATLOG_OBJECT_TYPE_GUARDIAN		= COMBATLOG_OBJECT_TYPE_GUARDIAN or 0x00002000
COMBATLOG_OBJECT_TYPE_NPC 		= COMBATLOG_OBJECT_TYPE_NPC or 0x0000800
PET_FILTER_BITMASK			= COMBATLOG_OBJECT_TYPE_PET + COMBATLOG_OBJECT_TYPE_GUARDIAN + COMBATLOG_OBJECT_TYPE_NPC
COMBATLOG_OBJECT_REACTION_FRIENDLY	= COMBATLOG_OBJECT_REACTION_FRIENDLY or 0x00000010


function HardFinger.UpdateGUIVariables()
	HardFinger.DPrint("HardFinger - GUI Options Updated")

	HardFinger.ListOfChannels = {}
	HardFinger.ListOfChannels["raid/party/instance"] = 1
	HardFinger.ListOfChannels["chat /print (you only)"] = false
	if IsInGuild() then
			HardFinger.ListOfChannels["/g GuildChat"] = "GUILD_CHANNEL"
	end
	for i=5,15,1 do
		a,b = GetChannelName(i)
		if a ~= 0 and b ~= 0 then
			HardFinger.ListOfChannels[a .. ". " .. b] = a
		end
	end
end

function HardFinger.ForceLoad(updateOnly)
	if updateOnly ~= true then
		HFDB = HFDB or { }

		HardFinger.HardFinger_Config.Profiles = HFDB

		HFDB.Default = { }
		HFDB.Default.Name = "Default"
		HardFinger.DefaultVariables = {"Status", "WhoPulled", "WhoPulledChannel", "Taunt", "BloodLust", "BattleRez", "TimeToDie", "ReportChannel", "Interrupt"}
		for _, v in pairs(HardFinger.DefaultVariables) do
			HFDB.Default[v] = false
		end

		HFDB.Default["EncountersOptions"] = {}
		for bossName, bossTable in pairs(Encounters_Options) do
			HFDB.Default["EncountersOptions"][bossName] = {}
			for id, bossAbilitiesTable in pairs(bossTable) do
				varName = bossAbilitiesTable.var
				HFDB.Default["EncountersOptions"][bossName][varName] = false
			end
		end
	
		-- Does the player has a valid active Profile selected?
		if HFDB[playerName] == nil or HFDB[playerName]["EncountersOptions"] == nil then
			HFDB[playerName] = HFDB.Default
		end
		
		-- Does the player has a valid active encounter options table?
		HFDB[playerName]["EncountersOptions"] = HFDB[playerName]["EncountersOptions"] or {}
		for bossName, bossTable in pairs(Encounters_Options) do
			HFDB[playerName]["EncountersOptions"][bossName] = HFDB[playerName]["EncountersOptions"][bossName] or {}
			for id, bossAbilitiesTable in pairs(bossTable) do
				varName = bossAbilitiesTable.var
				HFDB[playerName]["EncountersOptions"][bossName][varName] = HFDB[playerName]["EncountersOptions"][bossName][varName] or false
			end
		end
		
		loaded = true	
	end


	for _, v in pairs(HardFinger.DefaultVariables) do
		HardFinger.HardFinger_Config[v] = HFDB[playerName][v]
	end
	
	HardFinger.HardFinger_Config.EncountersOptions = HardFinger.TableHardCopy(HFDB[playerName].EncountersOptions)

	if (HardFingerOptions:IsShown()) then
		HardFingerOptions:Hide();
		HardFingerOptions:Show();
	end
end


function HardFinger.SaveVariables()
	for _, v in pairs(HardFinger.DefaultVariables) do
		HFDB[playerName][v] = HardFinger.HardFinger_Config[v]
	end
	HFDB[playerName].EncountersOptions = HardFinger.TableHardCopy(HardFinger.HardFinger_Config.EncountersOptions)
	HardFinger.ForceLoad(true)
end

function HardFinger.ResetProfile()
	HardFinger.DPrint("HardFinger - Profile: "..playerName.." reseted.")
	for _, v in pairs(HardFinger.DefaultVariables) do
		HFDB[playerName][v] = false
	end
	HFDB[playerName]["EncountersOptions"] = {}
	for bossName, bossTable in pairs(Encounters_Options) do
		HFDB[playerName]["EncountersOptions"][bossName] = {}
		for id, bossAbilitiesTable in pairs(bossTable) do
			
			varName = bossAbilitiesTable.var
			HardFinger.DPrint(bossName, varName)
			HFDB[playerName]["EncountersOptions"][bossName][varName] = false
		end
	end
	HardFinger.ForceLoad(true)
end


--
local function OnEvent(self, event, ...)
	if event == "COMBAT_LOG_EVENT_UNFILTERED" and P.Status == true then
		_, message, _, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags, spellID, spellName, param12, param13, param14, param15, _, _, param18 = ...
		-- Always running Display failTable time
		if HardFinger.FailTableDelayTimer ~= nil then
			HardFinger.DisplayFailTable()
		end
		HardFinger.DelayedTimer("Update")

		-- Summoning tracker for GetPetOwner function
		if message == "SPELL_SUMMON" then
			if bit.band(destFlags, PET_FILTER_BITMASK) ~= 0 then
				HardFinger.PetOwnersTable[destGUID] = {sourceName, sourceGUID}
			end
		end

		if P.WhoPulled == true then
			HardFinger.WhoPulled("ALL", false, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
		end
		
		-- Functions WhoTaunted, WhoREzzed and WhoLusted
		HardFinger.TauntRezLust(message, sourceName, sourceRaidFlags, destName, destRaidFlags, spellID)

		EO = "EncountersOptions"
			----------------------------------------------------------------------------------------------------------
			--						TRIAL OF VALOR BOSSES															--
			----------------------------------------------------------------------------------------------------------
		if HardFinger.ProperTrigger == "TOV" then
			-- Odyn
			if HardFinger.ActiveEncounter == 1958 then
				if P[EO]["Odyn"]["trackWhoPulled"] == true then
					HardFinger.WhoPulled("Hymdall|Hyrja", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID, "OdynEncounter")
				end
				
				HardFinger.BadSpellHitID(227712, 0, "Tank|Healer|Damager", false, P[EO][HardFinger.EncounterName]["SpearOfLightHits"], message, destName, destFlags, spellID, param13, param14, param15, param18) -- Spear of Light
				
			-- Guarm
			elseif HardFinger.ActiveEncounter == 1962 then
				if P[EO]["Guarm"]["trackWhoPulled"] == true then
					HardFinger.WhoPulled("Guarm", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
				end
				if P[EO]["Guarm"]["MixedElementsDeath"] then
					HardFinger.BadSpellHitIDDeath(227720, 0, "ALL", P[EO]["Guarm"]["MixedElementsDeathAction"], P[EO]["Guarm"]["MixedElementsDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Mixed Elements
					HardFinger.BadSpellHitIDDeath(227721, 0, "ALL", P[EO]["Guarm"]["MixedElementsDeathAction"], P[EO]["Guarm"]["MixedElementsDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Mixed Elements
					HardFinger.BadSpellHitIDDeath(227735, 0, "ALL", P[EO]["Guarm"]["MixedElementsDeathAction"], P[EO]["Guarm"]["MixedElementsDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Mixed Elements
				end
				if P[EO]["Guarm"]["TrampleHits"] then
					HardFinger.BadSpellHitID(227843, 0, "ALL", P[EO]["Guarm"]["TrampleHitsAction"], P[EO]["Guarm"]["TrampleHitsAction"], message, destName, destFlags, spellID, param13, param14, param15, param18)-- Trample
				end
				if P[EO]["Guarm"]["TrampleDeath"] then
					HardFinger.BadSpellHitIDDeath(227843, 0, "ALL", P[EO]["Guarm"]["TrampleDeathAction"], P[EO]["Guarm"]["TrampleDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Trample
				end
				if P[EO]["Guarm"]["FlashingFangsHits"] then
					HardFinger.BadSpellHitID(227514, 0, "Healer|Damager", P[EO]["Guarm"]["FlashingFangsHitsAction"], P[EO]["Guarm"]["FlashingFangsHitsAction"], message, destName, destFlags, spellID, param13, param14, param15, param18)-- Trample
				end
				if P[EO]["Guarm"]["FlashingFangsDeath"] then
					HardFinger.BadSpellHitIDDeath(227514, 0, "Healer|Damager", P[EO]["Guarm"]["FlashingFangsDeathAction"], P[EO]["Guarm"]["FlashingFangsDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Trample
				end
				if P[EO]["Guarm"]["BreathSoak"] then
					HardFinger.GenerateTargetsTableSoakID("SPELL_CAST_SUCCESS", 227573, nil, nil, "Tank|Healer|Damager", nil, message, spellID)
					HardFinger.RemoveFromTableIDbyEvent("SPELL_DAMAGE|SPELL_MISSED|SPELL_ABSORBED", 232777, message, destGUID, spellID)-- Fiery Phlegm ID: 232777
					HardFinger.RemoveFromTableIDbyEvent("SPELL_DAMAGE|SPELL_MISSED|SPELL_ABSORBED", 232798, message, destGUID, spellID)-- Salty Spittle ID: 232798
					HardFinger.RemoveFromTableIDbyEvent("SPELL_DAMAGE|SPELL_MISSED|SPELL_ABSORBED", 232800, message, destGUID, spellID)-- Dark Discharge ID: 232800
					HardFinger.DisplayTable("SPELL_CAST_SUCCESS"..227573, 227573, "SPELL_CAST_SUCCESS", 227658, " Failed to Soak:", 7, true, true, message, spellID)
				end
			-- Helya
			elseif HardFinger.ActiveEncounter == 2008 then
				if P[EO]["Helya"]["trackWhoPulled"] == true then
					HardFinger.WhoPulled("Helya", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
				end
				if P[EO]["Helya"]["DamageDoneBilewater"] == true then
					HardFinger.DamageToTarget("Bilewater Slime", "P1 - ", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
				end
				if P[EO]["Helya"]["OrbsDeath"] == true then
					HardFinger.BadSpellHitIDDeath(227930, 0, "ALL", P[EO]["Helya"]["OrbsDeathAction"], P[EO]["Helya"]["OrbsDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Orb of Corruption
					HardFinger.BadSpellHitIDDeath(228063, 0, "ALL", P[EO]["Helya"]["OrbsDeathAction"], P[EO]["Helya"]["OrbsDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Orb of Corrosion
					HardFinger.BadSpellHitIDDeath(228068, 0, "ALL", P[EO]["Helya"]["OrbsDeathAction"], P[EO]["Helya"]["OrbsDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Corrupted Slicer (Mythic P1 Orb of Corruption Link)
					HardFinger.BadSpellHitIDDeath(231416, 0, "ALL", P[EO]["Helya"]["OrbsDeathAction"], P[EO]["Helya"]["OrbsDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Corrosion Slicer (Mythic P3 Orb of Corrosion Link)
				end
				if P[EO]["Helya"]["TaintOfSeaHit"] == true then
					HardFinger.BadSpellHitID(228051, 0, "Healer|Damager", P[EO]["Helya"]["TaintOfSeaHitAction"], P[EO]["Helya"]["TaintOfSeaHitAction"], message, destName, destFlags, spellID, param13, param14, param15, param18)-- Taint Of Sea (Green puddle after dispel)
				end
				if P[EO]["Helya"]["TaintOfSeaDeath"] == true then
					HardFinger.BadSpellHitIDDeath(228051, 0, "Tank|Healer|Damager", P[EO]["Helya"]["TaintOfSeaDeathAction"], P[EO]["Helya"]["TaintOfSeaDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Taint Of Sea (Green puddle after dispel)
				end
				if P[EO]["Helya"]["BreathsHits"] == true then
					HardFinger.BadSpellHitID(230229, 0, "Healer|Damager", P[EO]["Helya"]["BreathsHitsAction"], P[EO]["Helya"]["BreathsHitsAction"], message, destName, destFlags, spellID, param13, param14, param15, param18)-- Bilewater Breath
					HardFinger.BadSpellHitID(228566, 0, "Healer|Damager", P[EO]["Helya"]["BreathsHitsAction"], P[EO]["Helya"]["BreathsHitsAction"], message, destName, destFlags, spellID, param13, param14, param15, param18)-- Corrupted Breath
				end
				if P[EO]["Helya"]["BreathsDeath"] == true then
					HardFinger.BadSpellHitIDDeath(230229, 0, "Tank|Healer|Damager", P[EO]["Helya"]["BreathsDeathAction"], P[EO]["Helya"]["BreathsDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Bilewater Breath
					HardFinger.BadSpellHitIDDeath(228566, 0, "Tank|Healer|Damager", P[EO]["Helya"]["BreathsDeathAction"], P[EO]["Helya"]["BreathsDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Corrupted Breath
				end
				if P[EO]["Helya"]["AnchorSlamHits"] == true then
					HardFinger.BadSpellHitID(228519, 0, "Healer|Damager", P[EO]["Helya"]["AnchorSlamHitsAction"], P[EO]["Helya"]["AnchorSlamHitsAction"], message, destName, destFlags, spellID, param13, param14, param15, param18)-- Grime Lord - Anchor Slam
				end
				if P[EO]["Helya"]["AnchorSlamDeath"] == true then
					HardFinger.BadSpellHitIDDeath(228519, 0, "Tank|Healer|Damager", P[EO]["Helya"]["AnchorSlamDeathAction"], P[EO]["Helya"]["AnchorSlamDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Grime Lord - Anchor Slam
				end
				if P[EO]["Helya"]["SludgeNovaHits"] == true then
					HardFinger.BadSpellHitID(228390, 0, "Tank|Healer|Damager", P[EO]["Helya"]["SludgeNovaHitsAction"], P[EO]["Helya"]["SludgeNovaHitsAction"], message, destName, destFlags, spellID, param13, param14, param15, param18)-- Grime Lord - Sludge Nova
				end
				if P[EO]["Helya"]["SludgeNovaDeath"] == true then
					HardFinger.BadSpellHitIDDeath(228390, 0, "Tank|Healer|Damager", P[EO]["Helya"]["SludgeNovaDeathAction"], P[EO]["Helya"]["SludgeNovaDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Grime Lord - Sludge Nova
				end
				if P[EO]["Helya"]["RotSpread"] == true then
					HardFinger.BadSpellAuraSpreadID(193367, "Grimelord", -1, P[EO]["Helya"]["RotSpreadAction"], P[EO]["Helya"]["RotSpreadAction"], message, spellID, sourceName, destName) -- Grime Lord - Rot
				end
				
			end
			----------------------------------------------------------------------------------------------------------
			--						NIGHTHOLD BOSSES																--
			----------------------------------------------------------------------------------------------------------
		elseif HardFinger.ProperTrigger == "NH" then
			-- Skorpyron
			if HardFinger.ActiveEncounter == 1849 then
				if P[EO][HardFinger.EncounterName]["trackWhoPulled"] == true then
					HardFinger.WhoPulled(HardFinger.EncounterName, true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
				end
				
				if P[EO][HardFinger.EncounterName]["trackShockwave"] == true then
					HardFinger.BuffDebuffAfterEvent(204316, "SPELL_CAST_SUCCESS", "ALL", UnitDebuff, "Broken Shard", 210074, "Total times hit: ", P[EO][HardFinger.EncounterName]["ShockwaveAction"], P[EO][HardFinger.EncounterName]["ShockwaveAction"], message, spellID)
				end
				
				if P[EO][HardFinger.EncounterName]["trackArcanoslashDamage"] == true then
					HardFinger.BadSpellDamageID(204275, 0, "Healer|Damager", false, true, message, destName, destFlags, spellID, param13, param14, param15, param18)-- Arcanoslash
				end
				if P[EO][HardFinger.EncounterName]["trackArcanoslashDeaths"] == true then
					HardFinger.BadSpellHitIDDeath(204275, 0, "Healer|Damager", false, true, message, destName, spellID, param13, param14, param15, param18)-- Arcanoslash
				end
				if P[EO][HardFinger.EncounterName]["trackBlast"] == true then
					HardFinger.BadSpellHitID(204483, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param15, param18)-- Focused Blast 204483
					HardFinger.BadSpellHitIDDeath(204483, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param18)-- Focused Blast 204483
				end
				if P[EO][HardFinger.EncounterName]["trackChitin"] == true then
					HardFinger.BadSpellDamageID(204744, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param15, param18)-- Green Toxic Pool
					HardFinger.BadSpellDamageID(204744, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param15, param18)-- Green Toxic Pool
				end
				if P[EO][HardFinger.EncounterName]["trackRedCrystalPulseDamage"] == true then
					HardFinger.BadSpellDamageID(214662, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param15, param18)-- Red pulsing
					HardFinger.BadSpellDamageID(211801, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param15, param18)-- Red pulsing
				end
				if P[EO][HardFinger.EncounterName]["trackRedCrystalPulseDeaths"] == true then
					HardFinger.BadSpellHitIDDeath(214662, 0, "ALL", false, true, sourceName, destName, spellID, param13, param14, param18)-- Red pulsing
					HardFinger.BadSpellHitIDDeath(211801, 0, "ALL", false, true, sourceName, destName, spellID, param13, param14, param18)-- Red pulsing
				end
				if P[EO][HardFinger.EncounterName]["trackRedCrystalExplosionDamage"] == true then
					HardFinger.BadSpellDamageID(226229, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param15, param18)-- Red crystal explosion // Volatile Resonance
				end
				if P[EO][HardFinger.EncounterName]["trackRedCrystalExplosionDeaths"] == true then
					HardFinger.BadSpellHitIDDeath(226229, 0, "ALL", false, true, sourceName, destName, spellID, param13, param14, param18)-- Red crystal explosion // Volatile Resonance
				end
				
				-- Chronomatic Anomaly / ID: 
			elseif HardFinger.ActiveEncounter == 1865 then
				if P[EO]["Chronomatic Anomaly"]["trackWhoPulled"] == true then
					HardFinger.WhoPulled("Chronomatic Anomaly", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
				end
				if P[EO]["Chronomatic Anomaly"]["trackTimeReleaseHealing"] == true then
					HardFinger.HealAbsorbed(message, "Time Release", sourceName, destName, param15)
				end
				if P[EO]["Chronomatic Anomaly"]["TimeReleaseExpiration"] == true then
					--HardFinger.AddToTargetsTableSoakID("SPELL_AURA_APPLIED", 206609, 206609, "Tank|Healer|Damager", nil, message, destName, spellID)-- Main Time Release ID: 206609, r/y/g IDs: 219966/219965/219964
					HardFinger.GenerateTargetsTableSoakID("SPELL_AURA_APPLIED", 206609, UnitDebuff, 206609, "Tank|Healer|Damager", 0.5, message, spellID)-- Time Release ID: 206609
					HardFinger.RemoveFromTableIDbyEvent("SPELL_AURA_APPLIED"..206609, "SPELL_AURA_REMOVED", 206609, message, destGUID, spellID)-- Main Time Release ID: 206609, r/y/g IDs: 219966/219965/219964
					HardFinger.DisplayTable("SPELL_AURA_APPLIED"..206609, "SPELL_DAMAGE|SPELL_MISSED|SPELL_ABSORBED", 206608, 206609, " Expired from:", 1, P[EO]["Chronomatic Anomaly"]["TimeReleaseExpirationAction"], P[EO]["Chronomatic Anomaly"]["TimeReleaseExpirationAction"], message, spellName, spellID) -- Time Release Damage ID: 226608
				end
				if P[EO]["Chronomatic Anomaly"]["trackBurstOfTimeDamage"] == true then
					HardFinger.BadSpellDamageID(206612, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param15, param18)-- Burst of Time
				end
				if P[EO]["Chronomatic Anomaly"]["trackTemporalOrbDamage"] == true then
					HardFinger.BadSpellDamageID(205707, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param15, param18)-- Temporal Orb
				end
				if HardFinger.Hpp("boss1") < 75 then
					HardFinger.BadSpreadDamageID(206615, "a", 0, "Damager|Healer", false, true, sourceName, destName, spellID, param13, param14, param18)
					HardFinger.BadSpellAuraID(206617, -1, true, false, true, message, spellID, destName, param14)-- Time Bomb Applications
				end
				if P[EO]["Chronomatic Anomaly"]["trackDamageDoneAdd"] == true and HardFinger.Hpp("boss1") < 30 then
					HardFinger.DamageToTarget("Waning Time Particle", "Last Set of Adds", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
				end
				
				HardFinger.BadSpellHitIDDeath(206612, 0, "ALL", P[EO]["Chronomatic Anomaly"]["announceBurstOfTimeDeaths"], P[EO]["Chronomatic Anomaly"]["trackBurstOfTimeDeaths"], message, destName, destFlags, spellID, param13, param14, param18)-- Burst of Time
				HardFinger.BadSpellHitIDDeath(205707, 0, "ALL", P[EO]["Chronomatic Anomaly"]["announceTemporalOrb"], P[EO]["Chronomatic Anomaly"]["trackTemporalOrbDeaths"], message, destName, destFlags, spellID, param13, param14, param18)-- Temporal Orb
				
				-- Trilliax / ID: 
			elseif HardFinger.ActiveEncounter == 1867 or HardFinger.LastEncounter == 1867 then
				if P[EO]["Trilliax"]["trackWhoPulled"] == true then
					HardFinger.WhoPulled("Trilliax", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
				end
				
				HardFinger.BadSpellHitIDDeath(207631, 0, "ALL", P[EO]["Trilliax"]["announceBeamDeaths"], P[EO]["Trilliax"]["trackBeamDeaths"], message, destName, spellID, param13, param14, param18)-- Annihilation
				
				if P[EO]["Trilliax"]["trackSterilize"] == true then
					HardFinger.SterilizeUptime(message, destName, spellID)
				end
				if P[EO]["Trilliax"]["trackCakes"] == true then
					HardFinger.BadSpellAuraID(214573, -1, nil, false, true,  message, destName, spellID, param14)-- Stuffed
				end
				if (GetTime() - HardFinger.EncounterTimer) < 280 then -- First add at 2:30, second add at 4:50
					if P[EO]["Trilliax"]["trackDamage1stImprint"] == true then
						HardFinger.DamageToTarget("Trilliax's Imprint", "First Add - ", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
					end
				else
					if P[EO]["Trilliax"]["trackDamage2ndImprint"] == true then
						HardFinger.DamageToTarget("Trilliax's Imprint", "Second Add - ", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
					end
				end			
				
				HardFinger.HelpfulSoakID(207328, 0, "ALL", false, P[EO]["Trilliax"]["CleansingDestructionSoak"], message, destName, destFlags, spellID, param13, param14, param15, param18)-- Cleansing Destruction Soak by jumping/clicking on it
				
				-- Spellblade Aluriel / ID: 
			elseif HardFinger.ActiveEncounter == 1871 then
				if P[EO]["Spellblade Aluriel"]["trackWhoPulled"] == true then
					HardFinger.WhoPulled("Spellblade Aluriel", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
				end
				if P[EO]["Spellblade Aluriel"]["trackIceBlock"] == true then
					HardFinger.BadSpellAuraID(213621, -1, nil, false, true, message, destName, spellID, param14) -- Frozen Debuff
				end
				if P[EO]["Spellblade Aluriel"]["damageDoneAddFire"] == true then
					HardFinger.DamageToTarget("Fiery Enchantment", "!", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
				end
				if P[EO]["Spellblade Aluriel"]["damageDoneAddArcane"] == true then
					HardFinger.DamageToTarget("Icy Enchantment", "!", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
				end
				if P[EO]["Spellblade Aluriel"]["damageDoneAddFrost"] == true then
					HardFinger.DamageToTarget("Arcane Enchantment", "!", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
				end
				
				HardFinger.BadSpellHitIDDeath(212494, 0, "Healer|Damager", P[EO]["Spellblade Aluriel"]["announceAnnihilateDeaths"], P[EO]["Spellblade Aluriel"]["trackAnnihilateDeaths"], message, destName, destFlags, spellID, param13, param14, param18)-- Annihilated
				
				if P[EO]["Spellblade Aluriel"]["trackAnnihilateDamage"] == true then
					HardFinger.BadSpellDamageID(212494, 0, "Healer|Damager", false, true, message, destName, destFlags, spellID, param13, param14, param15, param18)-- Annihilated
				end
				if P[EO]["Spellblade Aluriel"]["trackPyroInterrupts"] == true then
					HardFinger.HasInterrupt(213281, message, spellID, sourceName, destName) -- Pyroblast Add Cast
				end
				
				-- Tichondrius / ID: 
			elseif HardFinger.ActiveEncounter == 1862 then
				local timeLapsed = math.floor(GetTime() - HardFinger.EncounterTimer)
				if P[EO][HardFinger.EncounterName]["trackWhoPulled"] == true then
					HardFinger.WhoPulled("Tichondrius", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
				end
				if P[EO][HardFinger.EncounterName]["CarrionNightmareHit"] == true then
					HardFinger.BadSpellHitID(215988, 0, "ALL", P[EO][HardFinger.EncounterName]["CarrionNightmareHitAction"], P[EO][HardFinger.EncounterName]["CarrionNightmareHitAction"], message, destName, destFlags, spellID, param13, param14, param15, param18)-- Carrion Nightmare
				end
				if P[EO][HardFinger.EncounterName]["CarrionNightmareDeath"] == true then
					HardFinger.BadSpellHitIDDeath(215988, 0, "ALL", P[EO]["Spellblade Aluriel"]["CarrionNightmareDeathAction"], P[EO]["Spellblade Aluriel"]["CarrionNightmareDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Carrion Nightmare
				end
				if P[EO][HardFinger.EncounterName]["EchoesOfTheVoidDamage"] == true then
					HardFinger.BadSpellDamageID(213534, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param15, param18)-- Echoes of the Void
				end
				if P[EO][HardFinger.EncounterName]["EchoesOfTheVoidDeath"] == true then
					HardFinger.BadSpellHitIDDeath(213534, 0, "ALL", P[EO][HardFinger.EncounterName]["EchoesOfTheVoidDeathAction"], P[EO][HardFinger.EncounterName]["EchoesOfTheVoidDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Echoes of the Void
				end

				if P[EO][HardFinger.EncounterName]["EndOfNightDeath"] == true then
					HardFinger.BadSpellHitIDDeath(216688, 0, "ALL", P[EO][HardFinger.EncounterName]["EndOfNightDeathAction"], P[EO][HardFinger.EncounterName]["EndOfNightDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- End Of Night
				end
				
				if P[EO][HardFinger.EncounterName]["TrackFirstDamageDoneBats"] == true and timeLapsed < 200 then
					HardFinger.DamageToTarget("Phantasmal Bloodfang", "First P2 - ", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
				elseif P[EO][HardFinger.EncounterName]["TrackSecondDamageDoneBats"] == true and timeLapsed > 200 then
					HardFinger.DamageToTarget("Phantasmal Bloodfang", "Second P2 - ", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
				end
				
				if P[EO][HardFinger.EncounterName]["trackBatsDeaths"] == true then
					if message == "SPELL_CAST_SUCCESS" and spellID == 206365 then
						SendBeastReportAt = GetTime()+29
					end
					HardFinger.MobDeaths(message == "SPELL_CAST_START" and spellID == 206365, math.floor(GetTime() - (SendBeastReportAt or 0)) == 0, "Phantasmal Bloodfang", message, spellID, destName, param14)
				end
							
	
				-- Krosus / ID: 
			elseif HardFinger.ActiveEncounter == 1842 then
			
				if P[EO]["Krosus"]["trackWhoPulled"] == true then
					HardFinger.WhoPulled("Krosus", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
				end
				if P[EO]["Krosus"]["trackBeamTimesHit"] == true then
					HardFinger.BadSpellHitID(205391, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param15, param18)-- Fel Beam
				end
				if P[EO]["Krosus"]["trackBeamDeaths"] == true then
					HardFinger.BadSpellHitIDDeath(205391, 0, "ALL", P[EO]["Krosus"]["trackBeamDeathAnnounce"], true, message, destName, destFlags, spellID, param13, param14, param18)-- Fel Beam
				end
				if P[EO]["Krosus"]["trackBurningPitSoak"] == true then
					HardFinger.BadSpellHitID(206376, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param15, param18)-- Burning Spit
				end
				if P[EO]["Krosus"]["trackDamageToAdds"] == true then
					HardFinger.DamageToTarget("Burning Ember", "!", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
				end
				
				-- High Botanist Tel'arn / ID: 
			elseif HardFinger.ActiveEncounter == 1886 then
				bN = "High Botanist Tel'arn"
				if P[EO][bN]["trackWhoPulled"] == true then
					HardFinger.WhoPulled("High Botanist Tel'arn|Soralist Tel'arn|Arcanist Tel'arn|Naturalist Tel'arn", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID, "BotanistEncounter")
				end
				if P[EO][bN]["trackSolarDamage"] == true then
					HardFinger.BadSpellDamageID(218155, 0, "ALL", P[EO][bN]["SolarDamageAction"], P[EO][bN]["SolarDamageAction"], message, destName, destFlags, spellID, param13, param14, param15, param18)-- Solar Collapse
				end
				if P[EO][bN]["trackChaosDamage"] == true then
					HardFinger.BadSpellDamageID(218470, 0, "ALL", P[EO][bN]["ChaosDamageAction"], P[EO][bN]["ChaosDamageAction"], message, destName, destFlags, spellID, param13, param14, param15, param18)-- Controlled Chaos
					HardFinger.BadSpellDamageID(218466, 0, "ALL", P[EO][bN]["ChaosDamageAction"], P[EO][bN]["ChaosDamageAction"], message, destName, destFlags, spellID, param13, param14, param15, param18)-- Controlled Chaos
				end
				if P[EO][bN]["trackSolarDeath"] == true then
					HardFinger.BadSpellHitIDDeath(218155, 0, "ALL", P[EO][bN]["SolarDeathsAction"], P[EO][bN]["SolarDeathsAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Solar Collapse
				end
				if P[EO][bN]["trackChaosDeath"] == true then
					HardFinger.BadSpellHitIDDeath(218470, 0, "ALL", P[EO][bN]["ChaosDeathsAction"], P[EO][bN]["ChaosDeathsAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Controlled Chaos
					HardFinger.BadSpellHitIDDeath(218466, 0, "ALL", P[EO][bN]["ChaosDeathsAction"], P[EO][bN]["ChaosDeathsAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Controlled Chaos
				end
				
				HardFinger.BadSpellDamageID(219235, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param15, param18)-- Spores
				
				-- Star Augur Etraeus / ID: 
			elseif HardFinger.ActiveEncounter == 1863 then
				if P[EO]["Star Augur Etraeus"]["trackWhoPulled"] == true then
					HardFinger.WhoPulled("Star Augur Etraeus", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
				end

				HardFinger.BadSpellDamageID(206398, 0, "ALL", false, P[EO]["Star Augur Etraeus"]["TrackFelFlameDamage"], message, destName, destFlags, spellID, param13, param14, param15, param18)-- Fel Flame
				if P[EO]["Star Augur Etraeus"]["FelFlameDeath"] == true then
					HardFinger.BadSpellHitIDDeath(206398, 0, "ALL", P[EO]["Star Augur Etraeus"]["FelFlameDeathAction"], P[EO]["Star Augur Etraeus"]["FelFlameDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Fel Flame
				end
				
				if P[EO]["Star Augur Etraeus"]["NovasDeath"] == true then
					HardFinger.BadSpellHitIDDeath(206517, 2000000, "ALL", P[EO]["Star Augur Etraeus"]["NovasDeathAction"], P[EO]["Star Augur Etraeus"]["NovasDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Fel Nova
					HardFinger.BadSpellHitIDDeath(206954, 2000000, "ALL", P[EO]["Star Augur Etraeus"]["NovasDeathAction"], P[EO]["Star Augur Etraeus"]["NovasDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Frigid Nova
				end
				if P[EO]["Star Augur Etraeus"]["FelMeteorDeath"] == true then
					HardFinger.BadSpellHitIDDeath(216781, 0, "ALL", P[EO]["Star Augur Etraeus"]["FelMeteorDeathAction"], P[EO]["Star Augur Etraeus"]["FelMeteorDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Fel Ember (Mythic P2 Meteor)
				end
				if P[EO]["Star Augur Etraeus"]["FrigidPulseDeath"] == true then
					HardFinger.BadSpellHitIDDeath(216697, 0, "ALL", P[EO]["Star Augur Etraeus"]["FrigidPulseDeathAction"], P[EO]["Star Augur Etraeus"]["FrigidPulseDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Frigid Pulse
				end
				
				if P[EO][HardFinger.EncounterName]["WitnessTheVoid"] == true then
					HardFinger.BadSpellAuraID(207720, -1, nil, P[EO][HardFinger.EncounterName]["WitnessTheVoidAction"], P[EO][HardFinger.EncounterName]["WitnessTheVoidAction"], message, destName, spellID, param14)-- Witness the Void p3  
				end
				if P[EO][HardFinger.EncounterName]["trackDamageBigAdd"] == true then
					HardFinger.DamageToTarget("Thing That Should Not Be", "!", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
				end	
				if P[EO][HardFinger.EncounterName]["BigEyeBeamDeath"] == true then
					HardFinger.BadSpellHitIDDeath(216909, 0, "ALL", P[EO]["Star Augur Etraeus"]["BigEyeBeamDeathAction"], P[EO]["Star Augur Etraeus"]["BigEyeBeamDeathAction"], message, destName, destFlags, spellID, param13, param14, param18)-- World-Devouring Force
				end	
				
				-- Grand Magistrix Elisande / ID: 
			elseif HardFinger.ActiveEncounter == 1872 then
				if P[EO]["Elisande"]["trackWhoPulled"] == true then
					HardFinger.WhoPulled("Elisande", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
				end
				if P[EO]["Elisande"]["trackArcaneRingDamage"] == true then
					HardFinger.BadSpellDamageID(208659, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param15, param18)
				end
				HardFinger.BadSpellHitIDDeath(208659, 0, "ALL", P[EO]["Elisande"]["announceArcaneRingDeaths"], P[EO]["Elisande"]["trackArcaneRingDeaths"], message, destName, spellID, param13, param14, param18)
				if P[EO]["Elisande"]["trackAblativeInterrupt"] == true then
					HardFinger.HasInterrupt(209971, message, spellID, sourceName, destName)-- Ablative Pulse 209971
				end			
				-- Gul'dan / ID: 
			elseif HardFinger.ActiveEncounter == 1866 then
				if P[EO][HardFinger.EncounterName]["trackWhoPulled"] == true then
					HardFinger.WhoPulled("Gul'Dan", true, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
				end
				if P[EO][HardFinger.EncounterName]["FelEffluxDamage"] == true then
					HardFinger.BadSpellDamageID(206515, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param15, param18) -- Fel Efflux
				end
				if P[EO][HardFinger.EncounterName]["FelEffluxDeath"] == true then
					HardFinger.BadSpellHitIDDeath(206515, 0, "ALL", P[EO][HardFinger.EncounterName]["FelEffluxDeathAction"], P[EO][HardFinger.EncounterName]["FelEffluxDeathAction"], message, destName, spellID, param13, param14, param18) -- Fel Efflux
				end
				if P[EO][HardFinger.EncounterName]["HandOfGuldanDamage"] == true then
					HardFinger.BadSpellDamageID(212262, 0, "ALL", P[EO][HardFinger.EncounterName]["HandOfGuldanDamageAction"], P[EO][HardFinger.EncounterName]["HandOfGuldanDamageAction"], message, destName, destFlags, spellID, param13, param14, param15, param18)-- Hand of Guldan
				end
				if P[EO][HardFinger.EncounterName]["HandOfGuldanDeath"] == true then
					HardFinger.BadSpellHitIDDeath(212262, 0, "ALL", P[EO][HardFinger.EncounterName]["HandOfGuldanDeathAction"], P[EO][HardFinger.EncounterName]["HandOfGuldanDeathAction"], message, destName, spellID, param13, param14, param18) -- Hand of Guldan
				end
				if P[EO][HardFinger.EncounterName]["HandOfGuldanDeath"] == true then
					HardFinger.BadSpellHitID(227550, 0, "Healer|Damager", P[EO][HardFinger.EncounterName]["FelScytheHitAction"], P[EO][HardFinger.EncounterName]["FelScytheHitAction"], message, destName, destFlags, spellID, param13, param14, param15, param18)-- Fel Scythe
					HardFinger.BadSpellHitIDDeath(227550, 3000000, "ALL", P[EO][HardFinger.EncounterName]["FelScytheHitAction"], P[EO][HardFinger.EncounterName]["FelScytheHitAction"], message, destName, destFlags, spellID, param13, param14, param18)-- Fel Scythe
				end
				if P[EO][HardFinger.EncounterName]["trackEmpoweredBondsSoak"] == true then
					HardFinger.BadSpellHitID(206370, 0, "Healer|Damager", false, true, message, destName, destFlags, spellID, param13, param14, param15, param18)-- Empowered Bonds soak
				end
				if P[EO][HardFinger.EncounterName]["trackFelObeliskHit"] == true then
					HardFinger.BadSpellHitID(229948, 0, "ALL", false, true, message, destName, destFlags, spellID, param13, param14, param15, param18)-- Fel Obelisk
				end
				if P[EO][HardFinger.EncounterName]["trackDamageToEmpoweredEye"] == true then
					HardFinger.DamageToTarget("Empowered Eye of Gul'dan", "!", message, sourceGUID, sourceName, sourceFlags, destName, spellID, param13, param14, param18)
				end			
				if P[EO][HardFinger.EncounterName]["trackSouls"] == true then
					HardFinger.BadSpellAuraID(208802, -1, nil, false, true, message, destName, spellID, param14) -- Soul Corrosion
				end			
			end
		elseif HardFinger.ProperTrigger == "DUNGEON" then
			----------------------------------------------------------------------------------------------------------
			--						Halls of Valor
			----------------------------------------------------------------------------------------------------------
			HardFinger.HasInterrupt(215433, message, spellID, sourceName, destName) -- Holy Radience
			HardFinger.HasInterrupt(199726, message, spellID, sourceName, destName)	-- God-King Skolvand trash - Unruly Yell
			
			----------------------------------------------------------------------------------------------------------
			--						Eye of Azshara
			----------------------------------------------------------------------------------------------------------
			HardFinger.HasInterrupt(192135, message, spellID, sourceName, destName) -- Warlord Parjesh / Bellowing Roar
			HardFinger.HasInterrupt(197502, message, spellID, sourceName, destName)	-- Hatecoil Crestrider / Restoration
			
			----------------------------------------------------------------------------------------------------------
			--						Karazhan
			----------------------------------------------------------------------------------------------------------
			HardFinger.HasInterrupt(232115, message, spellID, sourceName, destName)	-- Firelands Portal
			HardFinger.HasInterrupt(229083, message, spellID, sourceName, destName)	-- Vizáduum the Watcher ID 2017, Burning Blast
			HardFinger.HasInterrupt(227592, message, spellID, sourceName, destName)	-- Medivh ID 1965 Frostbite
			HardFinger.HasInterrupt(228239, message, spellID, sourceName, destName)	-- Banshee fear, trash on the way to Curator - Terrifying Wail
			
			----------------------------------------------------------------------------------------------------------
			--						Court of Stars
			----------------------------------------------------------------------------------------------------------
			HardFinger.HasInterrupt(225100, message, spellID, sourceName, destName)	-- Charging Station
			HardFinger.HasInterrupt(208165, message, spellID, sourceName, destName)	-- Talixae Flamewreath - Withering Soul

			----------------------------------------------------------------------------------------------------------
			--						Vault of the Wardens
			----------------------------------------------------------------------------------------------------------
			HardFinger.HasInterrupt(196799, message, spellID, sourceName, destName)	-- Unleash Fury
		end
	elseif event == "PLAYER_ENTERING_WORLD" then
		HardFinger.DPrint("Entering World...")
		playerName = UnitName("player")
		HardFinger.CombatStatus = UnitAffectingCombat("player")
		HardFinger.WorldInfo()

	elseif event == "PLAYER_LOGOUT" or event == "PLAYER_LOGIN" then
		HardFinger.SaveVariables()

	elseif event == "ADDON_LOADED" then
		HardFinger.ForceLoad()	
	
	elseif event == "PLAYER_REGEN_ENABLED" then
		HardFinger.CombatStatus = false
		HardFinger.CombatToggle("OUT_OF_COMBAT")

	elseif event == "PLAYER_REGEN_DISABLED" then
		HardFinger.CombatStatus = true
		HardFinger.CombatToggle("IN_COMBAT")

	elseif event == "ENCOUNTER_START" then
		param1, param2 = ...
	    HardFinger.ActiveEncounter = param1
		HardFinger.EncounterTimer = GetTime()
		HardFinger.EncounterName = param2
		HardFinger.timestampDisplayTableTimed = nil
		HardFinger.DPrint(event, param1, param2)
		HardFinger.DisplayFailTable("HRESET")
		HardFinger.WorldInfo()
	elseif event == "ENCOUNTER_END" then
	    HardFinger.LastEncounter = HardFinger.ActiveEncounter
		HardFinger.ActiveEncounter = 0
		--HardFinger.EncounterName = "n/a"
		HardFinger.DPrint(event, encounterID, ...)
		HardFinger.DPrint("Requesting to display FailTable")
		HardFinger.DisplayFailTable("RESET")
		HardFinger.CombatStatus = false

	elseif event == "CHAT_MSG_ADDON" then
		prefix, message = ...
		if prefix == "D4" and string.find(message, "PT") then
			local index = string.find(message, "\t", 4) - 1
			local timer = string.sub(message, 4, index)
			HardFinger.DPrint(timer+GetTime())
			HardFinger.DBMPullTimer = timer+GetTime()
		end
	end
end

frame:SetScript("OnEvent", OnEvent)
SLASH_HARDFINGER1 = "/hf"
function SlashCmdList.HARDFINGER(msg, editbox)
	msg = string.lower(msg)
	if msg == "" then
		HardFinger.DPrint("\t Type...")
		HardFinger.DPrint("|cffff0000/hf stop |r - To pause the addon")
		HardFinger.DPrint("|cff00ff00/hf resume |r - To resume the addon")

		local loaded, reason = true
		if (loaded) == true then
			if (HardFingerOptions:IsShown()) then
				HardFingerOptions:Hide();
			else
				HardFinger.UpdateGUIVariables();
				HardFingerOptions:Show();
			end
		else
			AzMsg("Could not open HardFinger Options: |1"..tostring(reason).."|r. Please make sure the addon is enabled from the character selection screen.");
		end
	end
	if msg == "reset" then
		HardFinger.ResetProfile()
	elseif msg == "wp" or msg == "whopulled" then
		P.WhoPulled = HardFinger.toggletf(P.WhoPulled)
		HardFinger.DPrint("HardFinger - WhoPulled is "..HardFinger.StatusText(WhoPulled))
	elseif msg == "bloodlust" or msg == "lust" or msg == "bl" then
		P.BloodLust = HardFinger.toggletf(P.BloodLust)
		HardFinger.DPrint("HardFinger - BloodLust is "..HardFinger.StatusText(BloodLust))
	elseif msg == "ttd" or msg == "time to die" then
		P.TimeToDie = HardFinger.toggletf(P.TimeToDie)
		HardFinger.DPrint("HardFinger - TimeToDie is "..HardFinger.StatusText(TimeToDie))
	elseif msg == "battle rez" or msg == "battlerez" then
		P.BattleRez = HardFinger.toggletf(P.BattleRez)
		HardFinger.DPrint("HardFinger - BattleRez is "..HardFinger.StatusText(BattleRez))
	elseif msg == "taunt" then
		P.Taunt = HardFinger.toggletf(P.Taunt)
		HardFinger.DPrint("HardFinger - Taunt is "..HardFinger.StatusText(Taunt))
	elseif msg == "stop" then
		HardFinger.DPrint("HardFinger - Stopped.")
		P.Status = false
		frame:UnregisterAllEvents()
	elseif msg == "fl" then
		HardFinger.ForceLoad()
		HardFinger.DPrint("Hardfinger - Load forced")
	elseif msg == "sv" then
		HardFinger.SaveVariables()
		HardFinger.DPrint("Hardfinger - Saving Variables")	
	elseif msg == "dft" then
		HardFinger.DPrint("Hardfinger - Displaying Fail Table")
		HardFinger.DisplayFailTable()
	elseif msg == "wft" then
		HardFinger.DPrint("Hardfinger - Wiping Fail Table")
		wipe(HardFinger.FailTable)
	elseif msg == "d" then
		HardFinger.DPrint("HardFinger - Status Debugg")
		for _, v in pairs(DefaultVariables) do
			dbgt = P[v]
			if dbgt == nil then dbgt = "nil" end
			HardFinger.DPrint(v..": "..tostring(dbgt))
		end
	elseif msg == "resume" then
		HardFinger.DPrint("HardFinger - Resumed.")
		P.Status = true
		for _, v in pairs(EVENTS_TO_REGISTER) do
			frame:RegisterEvent(v)
		end
	end
end

-- Variables Loaded
function hf:VARIABLES_LOADED(event)
	isColorBlind = (GetCVar("colorblindMode") == "1");
	-- Default Fonts
	TT_DefaultConfig.fontFace = GameFontNormal:GetFont();
	TT_DefaultConfig.barFontFace = NumberFontNormal:GetFont();
	-- Init Config
	if (not TipTac_Config) then
		TipTac_Config = {};
	end
	cfg = setmetatable(TipTac_Config,{ __index = TT_DefaultConfig });
	-- Default the bar texture if it no longer exists
	bars[1]:SetStatusBarTexture(cfg.barTexture);
	if (not bars[1]:GetStatusBarTexture()) then
		cfg.barTexture = nil;
	end
	-- Hook Tips & Apply Settings
	self:HookTips();
	self:ApplySettings();
	-- Position
	if (cfg.left and cfg.top) then
		self:ClearAllPoints();
		self:SetPoint("TOPLEFT",UIParent,"BOTTOMLEFT",cfg.left,cfg.top);
	else
		self:Show();
		-- Just set left and top here, in case the player just closes the anchor without moving, so it doesn't keep showing up when they log in
		cfg.left, cfg.top = self:GetLeft(), self:GetTop();
	end
	-- Cleanup
	self:UnregisterEvent(event);
	self[event] = nil;
end